/*
 * @(#)src/classes/sov/java/net/URLEncoder.java, net, as142, 20060119 1.14.2.3
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 * IBM SDK, Java(tm) 2 Technology Edition, v1.4.2
 * (C) Copyright IBM Corp. 1998, 2004. All Rights Reserved
 * ===========================================================================
 */

/*
 * ===========================================================================
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * ===========================================================================
 */


/*
 * Change activity:
 *
 * Reason  Date   Origin   Description
 * ------  ----   ------   ----------------------------------------------------
 * 004235  310100 hdmaw    EBCDIC
 * 041015  110202 kwb      Remove 4235 optimization
 * 056111  301002 stalleyj Merge 1.4.1 changes 
 * 064147  211003 riclau   1.4.2 merge
 * 064147  061103 riclau   BufferedWriters needed because IBM use IO converters
 * 088171  120805 vbhargav 3% Regression in base runtiime running SpecJ due to 
 *                         ORB changes
 * 094241  190805 arvindr  JCK:api/java_net/URLEncoder/index.html#Encode fails
 *
 */

package java.net;

import java.io.ByteArrayOutputStream;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.BitSet;
import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.security.action.GetBooleanAction;
import sun.security.action.GetPropertyAction;

/**
 * Utility class for HTML form encoding. This class contains static methods
 * for converting a String to the <CODE>application/x-www-form-urlencoded</CODE> MIME
 * format. For more information about HTML form encoding, consult the HTML
 * <A HREF="http://www.w3.org/TR/html4/">specification</A>.
 *
 * <p>
 * When encoding a String, the following rules apply:
 *
 * <p>
 * <ul>
 * <li>The alphanumeric characters &quot;<code>a</code>&quot; through
 *     &quot;<code>z</code>&quot;, &quot;<code>A</code>&quot; through
 *     &quot;<code>Z</code>&quot; and &quot;<code>0</code>&quot;
 *     through &quot;<code>9</code>&quot; remain the same.
 * <li>The special characters &quot;<code>.</code>&quot;,
 *     &quot;<code>-</code>&quot;, &quot;<code>*</code>&quot;, and
 *     &quot;<code>_</code>&quot; remain the same.
 * <li>The space character &quot;<code>&nbsp;</code>&quot; is
 *     converted into a plus sign &quot;<code>+</code>&quot;.
 * <li>All other characters are unsafe and are first converted into
 *     one or more bytes using some encoding scheme. Then each byte is
 *     represented by the 3-character string
 *     &quot;<code>%<i>xy</i></code>&quot;, where <i>xy</i> is the
 *     two-digit hexadecimal representation of the byte.
 *     The recommended encoding scheme to use is UTF-8. However,
 *     for compatibility reasons, if an encoding is not specified,
 *     then the default encoding of the platform is used.
 * </ul>
 *
 * <p>
 * For example using UTF-8 as the encoding scheme the string &quot;The
 * string &#252;@foo-bar&quot; would get converted to
 * &quot;The+string+%C3%BC%40foo-bar&quot; because in UTF-8 the character
 * &#252; is encoded as two bytes C3 (hex) and BC (hex), and the
 * character @ is encoded as one byte 40 (hex).
 *
 * @author  Herb Jellinek
 * @version 1.28, 01/23/03
 * @since   JDK1.0
 */
public class URLEncoder {
    static BitSet dontNeedEncoding;
    static final int caseDiff = ('a' - 'A');
    static final String hexStr = "0123456789ABCDEF";                 //ibm.41015
    static String dfltEncName = null;
    static boolean isDfltEncShortCircuit = false;      /*ibm@88171*/
    
    static {

        /* The list of characters that are not encoded has been
         * determined as follows:
         *
         * RFC 2396 states:
         * -----
         * Data characters that are allowed in a URI but do not have a
         * reserved purpose are called unreserved.  These include upper
         * and lower case letters, decimal digits, and a limited set of
         * punctuation marks and symbols.
         *
         * unreserved  = alphanum | mark
         *
         * mark        = "-" | "_" | "." | "!" | "~" | "*" | "'" | "(" | ")"
         *
         * Unreserved characters can be escaped without changing the
         * semantics of the URI, but this should not be done unless the
         * URI is being used in a context that does not allow the
         * unescaped character to appear.
         * -----
         *
         * It appears that both Netscape and Internet Explorer escape
         * all special characters from this list with the exception
         * of "-", "_", ".", "*". While it is not clear why they are
         * escaping the other characters, perhaps it is safest to
         * assume that there might be contexts in which the others
         * are unsafe if not escaped. Therefore, we will use the same
         * list. It is also noteworthy that this is consistent with
         * O'Reilly's "HTML: The Definitive Guide" (page 164).
         *
         * As a last note, Intenet Explorer does not encode the "@"
         * character which is clearly not unreserved according to the
         * RFC. We are being consistent with the RFC in this matter,
         * as is Netscape.
         *
         */

        dontNeedEncoding = new BitSet(256);
        int i;
        for (i = 'a'; i <= 'z'; i++) {
            dontNeedEncoding.set(i);
        }
        for (i = 'A'; i <= 'Z'; i++) {
            dontNeedEncoding.set(i);
        }
        for (i = '0'; i <= '9'; i++) {
            dontNeedEncoding.set(i);
        }
        dontNeedEncoding.set(' '); /* encoding a space to a + is done
                                    * in the encode() method */
        dontNeedEncoding.set('-');
        dontNeedEncoding.set('_');
        dontNeedEncoding.set('.');
        dontNeedEncoding.set('*');

        dfltEncName = (String)AccessController.doPrivileged (
            new GetPropertyAction("file.encoding")
        );
        /*start of ibm@88171*/
        try {
            ByteArrayOutputStream buf = new ByteArrayOutputStream(1280);
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(buf, dfltEncName), 128);
            for (int c = 0; c < 128; c++) {
                writer.write((char)c);
            }
            writer.flush();
            byte[] ba = buf.toByteArray();
            if (ba.length == 128) {
                isDfltEncShortCircuit = true;
                for (int c = 0; c < 128; c++) {
                    if (ba[c] != c) {
                        isDfltEncShortCircuit = false;
                        break;
                    }
                }
            }
        } catch(Exception e) {
        }
        /*end of ibm@88171*/
    }

    /**
     * You can't call the constructor.
     */
    private URLEncoder() { }

    /**
     * Translates a string into <code>x-www-form-urlencoded</code>
     * format. This method uses the platform's default encoding
     * as the encoding scheme to obtain the bytes for unsafe characters.
     *
     * @param   s   <code>String</code> to be translated.
     * @deprecated The resulting string may vary depending on the platform's
     *             default encoding. Instead, use the encode(String,String)
     *             method to specify the encoding.
     * @return  the translated <code>String</code>.
     */
    public static String encode(String s) {

        String str = null;

        try {
            str = encode(s, dfltEncName);
        } catch (UnsupportedEncodingException e) {
            // The system should always have the platform default
        }

        return str;
    }

    /**
     * Translates a string into <code>application/x-www-form-urlencoded</code>
     * format using a specific encoding scheme. This method uses the
     * supplied encoding scheme to obtain the bytes for unsafe
     * characters.
     * <p>
     * <em><strong>Note:</strong> The <a href=
     * "http://www.w3.org/TR/html40/appendix/notes.html#non-ascii-chars">
     * World Wide Web Consortium Recommendation</a> states that
     * UTF-8 should be used. Not doing so may introduce
     * incompatibilites.</em>
     *
     * @param   s   <code>String</code> to be translated.
     * @param   enc   The name of a supported
     *    <a href="../lang/package-summary.html#charenc">character
     *    encoding</a>.
     * @return  the translated <code>String</code>.
     * @exception  UnsupportedEncodingException
     *             If the named encoding is not supported
     * @see URLDecoder#decode(java.lang.String, java.lang.String)
     * @since 1.4     
     */
    public static String encode(String s, String enc)
        throws UnsupportedEncodingException {

        boolean needToChange = false;
        boolean wroteUnencodedChar = true;                     /*ibm@88171*/
        int maxBytesPerChar = 10; // rather arbitrary limit, but safe for now
        /*start of ibm@88171*/
	StringBuffer out = null;		// lazy allocation if change is needed
        ByteArrayOutputStream buf = null;	// lazy allocation if conversion is required
        boolean firstTranslation = true;
        boolean mayRequirePrefix = true;
        BufferedWriter writer = null;
        boolean isISO88591 = enc.equalsIgnoreCase("iso8859-1") || enc.equalsIgnoreCase("iso-8859-1") ||
                             enc.equalsIgnoreCase("iso8859_1") || enc.equalsIgnoreCase("iso-8859_1");
        boolean isUTF8 = enc.equalsIgnoreCase("utf-8") || enc.equalsIgnoreCase("utf8");
        boolean isEncShortCircuit = isUTF8 || (isDfltEncShortCircuit && enc.equalsIgnoreCase(dfltEncName));
        /*end of ibm@88171*/
	/*start of ibm@94241*/
	OutputStreamWriter encoder = null;
	if ((enc == "") || (!isISO88591 && !isEncShortCircuit)) {
            buf = new ByteArrayOutputStream(maxBytesPerChar);
            encoder = new OutputStreamWriter(buf, enc);
	}
	/*end of ibm@94241*/
        
	for (int i = 0; i < s.length(); i++) {
            int c = (int) s.charAt(i);
            //System.out.println("Examining character: " + c);
            if (dontNeedEncoding.get(c)) {
                if (c == ' ') {
                    c = '+';
                    needToChange = true;
                }
                //System.out.println("Storing: " + c);
                /*start of ibm@88171*/
		if (needToChange) {
                  if (out == null) {
                      out = new StringBuffer(s.length());		// lazy allocation & fill content
                      for (int j = 0; j < i; j++) {
                          out.append((char)s.charAt(j));
                      }
                  }
		  /*end of ibm@88171*/
                out.append((char)c);
		/*start of ibm@88171*/
                }
                wroteUnencodedChar = mayRequirePrefix;
		/*end of ibm@88171*/
            } else {
                // convert to external encoding before hex conversion
		int wlen = 1;			     /*ibm@88171*/
                try {
                    /*start of ibm@88171*/
		    if (out == null) {
                        out = new StringBuffer(s.length()*3/2);		// lazy allocation & fill content
                        for (int j = 0; j < i; j++) {
                            out.append((char)s.charAt(j));
                        }
                    }
                    if ((isEncShortCircuit && (c <= '\u007f')) || (isISO88591 && (c < '\u00ff'))) {
                        out.append('%');
                        out.append(hexStr.charAt((c >> 4) & 0x0f));
                        out.append(hexStr.charAt(c&0x0f));
                        needToChange = true;
                        continue;
                    }
                    if (buf == null) {
                        buf = new ByteArrayOutputStream(maxBytesPerChar);
                    }
		    /*end of ibm@88171*/
                    if (wroteUnencodedChar) { // Fix for 4407610
                        /*start of ibm@94241*/
                        //writer = new BufferedWriter(new OutputStreamWriter(buf, enc), 2);/*ibm@88171*/
                        if (encoder == null) {
                            encoder = new OutputStreamWriter(buf, enc);
                        }
                        writer = new BufferedWriter(encoder, 2);
                        encoder = null;
                        /*end of ibm@94241*/
                        wroteUnencodedChar = false;
                    }
                    writer.write(c);
                    /*
                     * If this character represents the start of a Unicode
                     * surrogate pair, then pass in two characters. It's not
                     * clear what should be done if a bytes reserved in the
                     * surrogate pairs range occurs outside of a legal
                     * surrogate pair. For now, just treat it as if it were
                     * any other character.
                     */
                    if (c >= 0xD800 && c <= 0xDBFF) {
                        /*
                          System.out.println(Integer.toHexString(c)
                          + " is high surrogate");
                        */
                        if ( (i+1) < s.length()) {
                            int d = (int) s.charAt(i+1);
                            /*
                              System.out.println("\tExamining "
                              + Integer.toHexString(d));
                            */
                            if (d >= 0xDC00 && d <= 0xDFFF) {
                                /*
                                  System.out.println("\t"
                                  + Integer.toHexString(d)
                                  + " is low surrogate");
                                */
                                writer.write(d);
                                i++;
                                wlen++;			/*ibm@88171*/
                            }
                        }
                    }
                    writer.flush();
                } catch(IOException e) {
                    if (buf != null) {             /*ibm@88171*/
                    buf.reset();
                    }                              /*ibm@88171*/
                    continue;
                }
                byte[] ba = buf.toByteArray();
                /*start of ibm@88171*/
		int balen = ba.length;

                if (firstTranslation) {
                    if ((wlen == 1) && (balen == 1)) {
                        mayRequirePrefix = false;       // short cut for an obvious case (in length = out length)
                    } 
		    else {
                        try {
                            if (wlen == 1) {
                                writer.write((int) s.charAt(i));
                            } else {
                                writer.write((int) s.charAt(i-1));
                                writer.write((int) s.charAt(i));
                            }
                            writer.flush();
                        } catch(IOException e) {
                            buf.reset();
                            continue;
                        }
                        byte[] baagain = buf.toByteArray();
                        if (balen == (baagain.length-balen)) {
                            mayRequirePrefix = false;	// Lengths are same between first conversion and second conversion
                        } 
			else {
                        }
                      }
                    firstTranslation = false;
                }
		/*end of ibm@88171*/
                for (int j = 0; j < balen; j++) {            /*ibm@88171*/
                    out.append('%');
                    out.append(hexStr.charAt((ba[j] >> 4) & 0x0f));  //ibm.41015
                    out.append(hexStr.charAt(ba[j]&0x0f));           //ibm.41015
                }
                buf.reset();
                needToChange = true;
            }
        }

        return (needToChange? out.toString() : s);
    }
}
